package aexp.servicelist;

import java.util.ArrayList;
import java.util.List;

import android.app.ListActivity;
import android.app.NotificationManager;
import android.content.Context;
import android.os.Bundle;
import android.os.ServiceManagerNative;
import android.os.IServiceManager;
import android.os.IBinder;
import android.os.IInterface;
import android.os.IStatisticsService;
import android.os.DeadObjectException;
import android.os.SystemProperties;
import android.view.Menu;
import android.view.Menu.Item;
import android.widget.ArrayAdapter;
import android.view.View;
import android.widget.ListView;
import android.widget.ListAdapter;
import android.util.Log;
import java.lang.reflect.Field;

public class ServiceList extends ListActivity
{
    static final String LOG_TAG = "SERVICELIST";
	static final String ENDLIST_TEXT = "-- end list --";
    public static final int SERVICELIST_ID = Menu.FIRST;
    public static final int REPORT_ID = SERVICELIST_ID+1;
    public static final int ENABLESTATS_ID = SERVICELIST_ID+2;
    public static final int DISABLESTATS_ID = SERVICELIST_ID+3;
        
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle icicle)
    {
        super.onCreate(icicle);
        setContentView(R.layout.servicelist);
        getServiceList();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        boolean result = super.onCreateOptionsMenu(menu);
        menu.add(0, SERVICELIST_ID, R.string.servicelist);
	    menu.add(1, REPORT_ID, R.string.report );
        return result;
    }

    @Override
    public boolean onOptionsItemSelected(Item item) {
    	switch (item.getId()) {
    	  case SERVICELIST_ID:
    		getServiceList();
    		break;

		  case REPORT_ID:
			statserviceReport();
			break;
    	}
    	
        return super.onOptionsItemSelected(item);
    }
        
    @Override
    protected void onListItemClick(ListView l, View v, int position, long id) {
        super.onListItemClick(l, v, position, id);
		ListAdapter la = getListAdapter();
		String item = (String)la.getItem( position );
		if( item.equals( ENDLIST_TEXT ) )
			return;
		Log.d( LOG_TAG, "Item selected: "+item );
		String message = null;
		try {
          IServiceManager sm = ServiceManagerNative.getDefault();
		  IBinder sif = sm.getService( item );
		  message = sif.getClass().getName();
	      NotificationManager nm = (NotificationManager)
				getSystemService( Context.NOTIFICATION_SERVICE);
      	  nm.notifyWithText(123, message,
                    NotificationManager.LENGTH_LONG, null );
		} catch( DeadObjectException ex ) {
		  Log.e( LOG_TAG, "Cannot obtain IServiceManager (item select)" ); 
		}
    }

    private void getServiceList() {
		IServiceManager sm = null;
		String[] services = new String[0];
		try {
          sm = ServiceManagerNative.getDefault();
		  services = sm.listServices();
		} catch( DeadObjectException ex ) {
		  Log.e( LOG_TAG, "Cannot obtain IServiceManager" ); 
		}
        // We need a list of strings for the list items
        List<String> items = new ArrayList<String>();

        for ( int i = 0 ; i < services.length ; ++i )
			items.add( services[i] );
        items.add( "-- end list --" );
        // Now create an array adapter and set it to display using our row
        ArrayAdapter<String> notes = 
            new ArrayAdapter<String>(this, R.layout.service_row, items);
        setListAdapter(notes);        
    }

    private void statserviceReport() {
		try {
          IServiceManager sm = ServiceManagerNative.getDefault();
		  IBinder statBinder = sm.getService( "statistics" );
		  if( statBinder == null ) {
			Log.e( LOG_TAG, "Cannot obtain binder for statistics service" );
		    return;
		  }
		  IStatisticsService statService = 
			IStatisticsService.Stub.asInterface( statBinder );
		  if( statService == null ) {
			Log.e( LOG_TAG, "Cannot obtain statistics service interface" );
		    return;
		  }
		  Log.i( LOG_TAG,"ro.config.nocheckin: "+
				SystemProperties.get( "ro.config.nocheckin","" ) );
		  statService.report( "STAT1","20" );
		  Log.i( LOG_TAG,"Statistics reported" );
		} catch( DeadObjectException ex ) {
		  Log.e( LOG_TAG, "Cannot obtain IServiceManager" ); 
		}
    }
}
